@@ -4,14 +4,17 @@
 package ca.odell.glazedlists;
 
 // the Glazed Lists' change objects
-import ca.odell.glazedlists.event.*;
-// volatile implementation support
-import ca.odell.glazedlists.impl.*;
-// concurrency is similar to java.util.concurrent in J2SE 1.5
-import ca.odell.glazedlists.util.concurrent.*;
-// for iterators
-import java.util.*;
+import ca.odell.glazedlists.event.ListEventAssembler;
+import ca.odell.glazedlists.event.ListEventListener;
+import ca.odell.glazedlists.event.ListEventPublisher;
+import ca.odell.glazedlists.impl.EventListIterator;
+import ca.odell.glazedlists.impl.GlazedListsImpl;
+import ca.odell.glazedlists.impl.SimpleIterator;
+import ca.odell.glazedlists.impl.SubEventList;
+import ca.odell.glazedlists.util.concurrent.ReadWriteLock;
+
 import java.lang.reflect.Array;
+import java.util.*;
 
 /**
  * A convenience class that implements common functionality for all {@link EventList}s.
@@ -67,12 +70,12 @@ public abstract class AbstractEventList<E> implements EventList<E> {
     }
 
     /** {@inheritDoc} */
-    public void addListEventListener(ListEventListener<E> listChangeListener) {
+    public void addListEventListener(ListEventListener<? super E> listChangeListener) {
         updates.addListEventListener(listChangeListener);
     }
 
     /** {@inheritDoc} */
-    public void removeListEventListener(ListEventListener<E> listChangeListener) {
+    public void removeListEventListener(ListEventListener<? super E> listChangeListener) {
         updates.removeListEventListener(listChangeListener);
     }
 
@@ -363,11 +366,9 @@ public abstract class AbstractEventList<E> implements EventList<E> {
      */
     public boolean removeAll(Collection<?> values) {
         boolean changed = false;
-        for(Iterator i = values.iterator(); i.hasNext(); ) {
-            Object value = i.next();
-            int index = -1;
-            while((index = indexOf(value)) != -1) {
-                remove(index);
+        for(Iterator i = iterator(); i.hasNext(); ) {
+            if(values.contains(i.next())) {
+                i.remove();
                 changed = true;
             }
         }
