@@ -4,10 +4,12 @@
 package ca.odell.glazedlists;
 
 // the core Glazed Lists packages
-import ca.odell.glazedlists.event.*;
-import ca.odell.glazedlists.matchers.*;
-// volatile implementation support
-import ca.odell.glazedlists.impl.adt.*;
+import ca.odell.glazedlists.event.ListEvent;
+import ca.odell.glazedlists.impl.adt.Barcode;
+import ca.odell.glazedlists.impl.adt.BarcodeIterator;
+import ca.odell.glazedlists.matchers.Matcher;
+import ca.odell.glazedlists.matchers.MatcherEditor;
+import ca.odell.glazedlists.matchers.Matchers;
 
 /**
  * An {@link EventList} that shows a subset of the elements of a source
@@ -21,6 +23,10 @@ import ca.odell.glazedlists.impl.adt.*;
  * breaks the contract required by {@link java.util.List}. See {@link EventList}
  * for an example.
  *
+ * <p><strong><font color="#FF0000">Warning:</font></strong> This class is
+ * thread ready but not thread safe. See {@link EventList} for an example
+ * of thread safe code.
+ *
  * <p><table border="1" width="100%" cellpadding="3" cellspacing="0">
  * <tr class="TableHeadingColor"><td colspan=2><font size="+2"><b>EventList Overview</b></font></td></tr>
  * <tr><td class="TableSubHeadingColor"><b>Writable:</b></td><td>yes</td></tr>
@@ -194,7 +200,8 @@ public final class FilterList<E> extends TransformedList<E,E> {
 
                     // if this value was not filtered out, it is now so add a change
                     if(filteredIndex != -1) {
-                        updates.addDelete(filteredIndex);
+                        E removed = listChanges.getOldValue();
+                        updates.elementDeleted(filteredIndex, removed);
                     }
 
                     // remove this entry from the flag list
@@ -204,13 +211,14 @@ public final class FilterList<E> extends TransformedList<E,E> {
                 } else if(changeType == ListEvent.INSERT) {
 
                     // whether we should add this item
-                    boolean include = currentMatcher.matches(source.get(sourceIndex));
+                    E element = source.get(sourceIndex);
+                    boolean include = currentMatcher.matches(element);
 
                     // if this value should be included, add a change and add the item
                     if(include) {
                         flagList.addBlack(sourceIndex, 1);
                         int filteredIndex = flagList.getBlackIndex(sourceIndex);
-                        updates.addInsert(filteredIndex);
+                        updates.elementInserted(filteredIndex, element);
 
                     // if this value should not be included, just add the item
                     } else {
@@ -224,21 +232,22 @@ public final class FilterList<E> extends TransformedList<E,E> {
                     int filteredIndex = flagList.getBlackIndex(sourceIndex);
                     boolean wasIncluded = filteredIndex != -1;
                     // whether we should add this item
-                    boolean include = currentMatcher.matches(source.get(sourceIndex));
+                    E updated = source.get(sourceIndex);
+                    boolean include = currentMatcher.matches(updated);
 
                     // if this element is being removed as a result of the change
                     if(wasIncluded && !include) {
                         flagList.setWhite(sourceIndex, 1);
-                        updates.addDelete(filteredIndex);
+                        updates.elementDeleted(filteredIndex, listChanges.getOldValue());
 
                     // if this element is being added as a result of the change
                     } else if(!wasIncluded && include) {
                         flagList.setBlack(sourceIndex, 1);
-                        updates.addInsert(flagList.getBlackIndex(sourceIndex));
+                        updates.elementInserted(flagList.getBlackIndex(sourceIndex), updated);
 
                     // this element is still here
                     } else if(wasIncluded && include) {
-                        updates.addUpdate(filteredIndex);
+                        updates.elementUpdated(filteredIndex, listChanges.getOldValue(), updated);
 
                     }
                 }
@@ -291,8 +300,10 @@ public final class FilterList<E> extends TransformedList<E,E> {
         // all of these changes to this list happen "atomically"
         updates.beginEvent();
 
-        // filter out all remaining items in this list
-        if(size() > 0) updates.addDelete(0, size() - 1);
+        // fire all the elements in the list as deleted
+        for(int i = 0; i < size(); i++) {
+            updates.elementDeleted(0, get(i));
+        }
 
         // reset the flaglist to all white (which matches nothing)
         flagList.clear();
@@ -322,7 +333,8 @@ public final class FilterList<E> extends TransformedList<E,E> {
         // to black as we go so that flag offsets are correct
         for(BarcodeIterator i = flagList.iterator(); i.hasNextWhite();) {
             i.nextWhite();
-            updates.addInsert(i.getIndex());
+            int index = i.getIndex();
+            updates.elementInserted(index, source.get(index));
         }
         flagList.clear();
         flagList.addBlack(0, source.size());
@@ -343,8 +355,9 @@ public final class FilterList<E> extends TransformedList<E,E> {
         // for all filtered items, see what the change is
         for(BarcodeIterator i = flagList.iterator(); i.hasNextWhite();) {
             i.nextWhite();
-            if(currentMatcher.matches(source.get(i.getIndex()))) {
-                updates.addInsert(i.setBlack());
+            E element = source.get(i.getIndex());
+            if(currentMatcher.matches(element)) {
+                updates.elementInserted(i.setBlack(), element);
             }
         }
 
@@ -364,10 +377,11 @@ public final class FilterList<E> extends TransformedList<E,E> {
         // for all unfiltered items, see what the change is
         for(BarcodeIterator i = flagList.iterator(); i.hasNextBlack();) {
             i.nextBlack();
-            if(!currentMatcher.matches(source.get(i.getIndex()))) {
+            E value = source.get(i.getIndex());
+            if(!currentMatcher.matches(value)) {
                 int blackIndex = i.getBlackIndex();
                 i.setWhite();
-                updates.addDelete(blackIndex);
+                updates.elementDeleted(blackIndex, value);
             }
         }
 
@@ -391,16 +405,17 @@ public final class FilterList<E> extends TransformedList<E,E> {
             int filteredIndex = i.getBlackIndex();
             boolean wasIncluded = filteredIndex != -1;
             // whether we should add this item
-            boolean include = currentMatcher.matches(source.get(i.getIndex()));
+            E value = source.get(i.getIndex());
+            boolean include = currentMatcher.matches(value);
 
             // this element is being removed as a result of the change
             if(wasIncluded && !include) {
                 i.setWhite();
-                updates.addDelete(filteredIndex);
+                updates.elementDeleted(filteredIndex, value);
 
             // this element is being added as a result of the change
             } else if(!wasIncluded && include) {
-                updates.addInsert(i.setBlack());
+                updates.elementInserted(i.setBlack(), value);
             }
         }
 
