@@ -3,19 +3,22 @@
 /*                                                     O'Dell Engineering Ltd.*/
 package ca.odell.glazedlists;
 
-import java.util.*;
-// for weak reference proxying
 import ca.odell.glazedlists.event.ListEventListener;
-// for access to volatile classes
+import ca.odell.glazedlists.gui.AdvancedTableFormat;
+import ca.odell.glazedlists.gui.TableFormat;
+import ca.odell.glazedlists.gui.WritableTableFormat;
 import ca.odell.glazedlists.impl.*;
-import ca.odell.glazedlists.impl.filter.StringTextFilterator;
-import ca.odell.glazedlists.impl.sort.*;
+import ca.odell.glazedlists.impl.functions.ConstantFunction;
 import ca.odell.glazedlists.impl.beans.*;
-import ca.odell.glazedlists.impl.matchers.*;
-// implemented interfaces
-import ca.odell.glazedlists.gui.*;
-import ca.odell.glazedlists.matchers.*;
-import java.util.Comparator;
+import ca.odell.glazedlists.impl.filter.StringTextFilterator;
+import ca.odell.glazedlists.impl.matchers.FixedMatcherEditor;
+import ca.odell.glazedlists.impl.sort.*;
+import ca.odell.glazedlists.matchers.Matcher;
+import ca.odell.glazedlists.matchers.MatcherEditor;
+import ca.odell.glazedlists.matchers.Matchers;
+
+import java.beans.PropertyChangeEvent;
+import java.util.*;
 
 /**
  * A factory for creating all sorts of objects to be used with Glazed Lists.
@@ -129,12 +132,43 @@ public final class GlazedLists {
     private static Comparator<Comparable> reversedComparable = null;
 
     /**
-     * Creates a {@link Comparator} that uses Reflection to compare two instances
-     * of the specified {@link Class} by the given JavaBean property.  The JavaBean
-     * property must implement {@link Comparable}.
+     * Creates a {@link Comparator} that uses Reflection to compare two
+     * instances of the specified {@link Class} by the given JavaBean
+     * properties. The JavaBean <code>property</code> and any extra
+     * <code>properties</code> must implement {@link Comparable}.
+     *
+     * <p>The following example code sorts a List of Customers by first name,
+     * with ties broken by last name.
+     *
+     * <pre>
+     *    List<Customer> customers = ...
+     *    Comparator<Customer> comparator = GlazedLists.beanPropertyComparator(Customer.class, "firstName", "lastName");
+     *    Collections.sort(customers, comparator);
+     * </pre>
+     *
+     * @param clazz the name of the class which defines the accessor method
+     *      for the given <code>property</code> and optionaly <code>properties</code>
+     * @param property the name of the first Comparable property to be extracted
+     *      and used to compare instances of the <code>clazz</code>
+     * @param properties the name of optional Comparable properties, each of
+     *      which is used to break ties for the prior property.
      */
-    public static <T> Comparator<T> beanPropertyComparator(Class<T> className, String property) {
-        return beanPropertyComparator(className, property, comparableComparator());
+    public static <T> Comparator<T> beanPropertyComparator(Class<T> clazz, String property, String... properties) {
+        // build the Comparator that must exist
+        Comparator<T> firstComparator = beanPropertyComparator(clazz, property, comparableComparator());
+
+        // if only one Comparator is specified, return it immediately
+        if (properties.length == 0) return firstComparator;
+
+        // build the remaining Comparators
+        final List<Comparator<T>> comparators = new ArrayList<Comparator<T>>(properties.length+1);
+        comparators.add(firstComparator);
+
+        for (int i = 0; i < properties.length; i++)
+            comparators.add(beanPropertyComparator(clazz, properties[i], comparableComparator()));
+
+        // chain all Comparators together
+        return chainComparators(comparators);
     }
 
     /**
@@ -166,26 +200,35 @@ public final class GlazedLists {
     /**
      * Creates a chain of {@link Comparator}s that applies the provided
      * {@link Comparator}s in the sequence specified until differences or
-     * absoulute equality.is determined.
+     * absolute equality is determined.
      */
     public static <T> Comparator<T> chainComparators(List<Comparator<T>> comparators) {
         return new ComparatorChain<T>(comparators);
     }
 
+    /**
+     * Creates a chain of {@link Comparator}s that applies the provided
+     * {@link Comparator}s in the sequence specified until differences or
+     * absolute equality is determined.
+     */
+    public static <T> Comparator<T> chainComparators(Comparator<T>... comparators) {
+        return chainComparators(Arrays.asList(comparators));
+    }
+
     /**
      * Creates a {@link Comparator} that compares {@link Comparable} objects.
      */
-    public static Comparator<Comparable> comparableComparator() {
+    public static <T extends Comparable> Comparator<T> comparableComparator() {
         if(comparableComparator == null) comparableComparator = new ComparableComparator();
-        return comparableComparator;
+        return (Comparator<T>)comparableComparator;
     }
 
     /**
      * Creates a reverse {@link Comparator} that works for {@link Comparable} objects.
      */
-    public static Comparator<Comparable> reverseComparator() {
+    public static <T extends Comparable> Comparator<T> reverseComparator() {
         if(reversedComparable == null) reversedComparable = reverseComparator(comparableComparator());
-        return reversedComparable;
+        return (Comparator<T>)reversedComparable;
     }
 
     /**
@@ -246,13 +289,13 @@ public final class GlazedLists {
 
     // TextFilterators // // // // // // // // // // // // // // // // // // //
 
-    private static StringTextFilterator stringTextFilterator = null;
+    private static TextFilterator<Object> stringTextFilterator = null;
 
     /**
      * Creates a {@link TextFilterator} that searches the given JavaBean
      * properties.
      */
-    public static <E> TextFilterator<E> textFilterator(String[] propertyNames) {
+    public static <E> TextFilterator<E> textFilterator(String... propertyNames) {
         return new BeanTextFilterator<Object,E>(propertyNames);
     }
 
@@ -260,17 +303,33 @@ public final class GlazedLists {
      * Creates a {@link TextFilterator} that searches the given JavaBean
      * properties.
      */
-    public static <D,E> Filterator<D,E> filterator(String[] propertyNames) {
+    public static <E> TextFilterator<E> textFilterator(Class<E> beanClass, String... propertyNames) {
+        return new BeanTextFilterator<Object,E>(beanClass, propertyNames);
+    }
+
+    /**
+     * Creates a {@link TextFilterator} that searches the given JavaBean
+     * properties.
+     */
+    public static <D,E> Filterator<D,E> filterator(String... propertyNames) {
         return new BeanTextFilterator<D,E>(propertyNames);
     }
 
+    /**
+     * Creates a {@link TextFilterator} that searches the given JavaBean
+     * properties of the specified class.
+     */
+    public static <D,E> Filterator<D,E> filterator(Class<E> beanClass, String... propertyNames) {
+        return new BeanTextFilterator<D,E>(beanClass, propertyNames);
+    }
+
     /**
      * Creates a {@link TextFilterator} that searches against an Object's
      * {@link Object#toString() toString()} value.
      */
-    public static TextFilterator toStringTextFilterator() {
+    public static <E> TextFilterator<E> toStringTextFilterator() {
         if(stringTextFilterator == null) stringTextFilterator = new StringTextFilterator();
-        return stringTextFilterator;
+        return (TextFilterator<E>) stringTextFilterator;
     }
 
 
@@ -298,6 +357,13 @@ public final class GlazedLists {
 
     // EventLists // // // // // // // // // // // // // // // // // // // // //
 
+    /**
+     * Creates a new {@link EventList} which contains the given elements.
+     */
+    public static <E> EventList<E> eventListOf(E... contents) {
+        return eventList(contents == null ? Collections.EMPTY_LIST : Arrays.asList(contents));
+    }
+
     /**
      * Creates a new {@link EventList} which contains the contents of the specified
      * {@link Collection}. The {@link EventList}'s order will be determined by
@@ -363,25 +429,25 @@ public final class GlazedLists {
     }
 
     /**
-     * Provides a proxy to another ListEventListener that may go out of
-     * scope without explicitly removing itself from the source list's set of
+     * Provides a proxy to another ListEventListener that may go out of scope
+     * without explicitly removing itself from the source list's set of
      * listeners.
      *
      * <p>This exists to solve a garbage collection problem. Suppose I have an
-     * {@link EventList} <i>L</i> and I request a {@link ListIterator} for <i>L</i>.
+     * {@link EventList} <i>L</i> and I obtain a {@link ListIterator} for <i>L</i>.
      * The {@link ListIterator} must listen for change events to <i>L</i> in order
-     * to be consistent. Therefore such an iterator will register
-     * itself as a listener for <i>L</i>. When the iterator goes out of scope (as
-     * they usually do), it will remain as a listener for <i>L</i>. This prevents
-     * the iterator object from ever being garbage collected! But the iterator is
-     * never used again. Because iterators can be used very frequently, this will
+     * to be consistent. Therefore such an iterator will register itself as a
+     * listener for <i>L</i>. When the iterator goes out of scope (as they usually
+     * do), it will remain as a listener of <i>L</i>. This prevents the iterator
+     * object from ever being garbage collected, though the iterator can never be
+     * never used again! Because iterators can be used very frequently, this will
      * cause an unacceptable memory leak.
      *
      * <p>Instead of adding the iterator directly as a listener for <i>L</i>, add
-     * the proxy instead. The proxy will retain a <code>WeakReference</code> to the
+     * a proxy instead. The proxy will retain a <code>WeakReference</code> to the
      * iterator and forward events to the iterator as long as it is reachable. When
-     * theiterator is no longer reachable, the proxy will remove itself from the list
-     * of listeners for <i>L</i>. All garbage is then available for collection.
+     * the iterator is no longer reachable, the proxy will remove itself from the
+     * list of listeners for <i>L</i>. All garbage is then available for collection.
      *
      * @see java.lang.ref.WeakReference
      */
@@ -405,7 +471,61 @@ public final class GlazedLists {
      * @return an ObservableElementList.Connector for the specified class
      */
     public static <E> ObservableElementList.Connector<E> beanConnector(Class<E> beanClass) {
-        return new JavaBeanEventListConnector<E>(beanClass);
+        return new BeanConnector<E>(beanClass);
+    }
+    
+    /**
+     * Create a new Connector for the {@link ObservableElementList} that works with JavaBeans'
+     * {@link java.beans.PropertyChangeListener}. The methods to add and remove listeners are
+     * detected automatically by examining the bean class and searching for a method prefixed with
+     * "add" or "remove" taking a single {@link java.beans.PropertyChangeListener} argument.
+     * <p>Use this variant, if you want to control which {@link java.beans.PropertyChangeEvent}s
+     * are delivered to the ObservableElementList. You can match or filter events by name. 
+     * <p>If <code>matchPropertyNames</code> is <code>true</code>, the <code>propertyNames</code>
+     * parameter specifies the set of properties by name whose {@link java.beans.PropertyChangeEvent}s 
+     * should be delivered to the ObservableElementList, e.g. property change events for properties 
+     * not contained in the specified <code>propertyNames</code> are ignored in this case.
+     * If <code>matchPropertyNames</code> is <code>false</code>, then the specified 
+     * <code>propertyNames</code> are filtered, e.g. all but the specified property change events are
+     * delivered to the ObservableElementList.
+     * 
+     * @param beanClass a class with both
+     *        <code>addPropertyChangeListener(PropertyChangeListener)</code> and
+     *        <code>removePropertyChangeListener(PropertyChangeListener)</code>, or similar
+     *        methods.
+     * @param matchPropertyNames if <code>true</code>, match property change events against the
+     *        specified property names, if <code>false</code> filter them
+     * @param propertyNames specifies the properties by name whose {@link java.beans.PropertyChangeEvent}s
+     *        should be matched or filtered
+     * @return an ObservableElementList.Connector for the specified class
+     */
+    public static <E> ObservableElementList.Connector<E> beanConnector(Class<E> beanClass, boolean matchPropertyNames, String... propertyNames) {
+        final Matcher<PropertyChangeEvent> byNameMatcher = Matchers.propertyEventNameMatcher(matchPropertyNames, propertyNames);
+        return beanConnector(beanClass, byNameMatcher);
+    }
+    
+    /**
+     * Create a new Connector for the {@link ObservableElementList} that works with JavaBeans'
+     * {@link java.beans.PropertyChangeListener}. The methods to add and remove listeners are
+     * detected automatically by examining the bean class and searching for a method prefixed with
+     * "add" or "remove" taking a single {@link java.beans.PropertyChangeListener} argument.
+     * 
+     * <p> The event matcher allows filtering of {@link java.beans.PropertyChangeEvent}s. 
+     * Only matching events are delivered to the ObservableElementList. 
+     * To create a matcher that matches PropertyChangeEvents by property names, you can use
+     * {@link Matchers#propertyEventNameMatcher(boolean, String[])}
+     * 
+     * @param beanClass a class with both
+     *        <code>addPropertyChangeListener(PropertyChangeListener)</code> and
+     *        <code>removePropertyChangeListener(PropertyChangeListener)</code>, or similar
+     *        methods.
+     * @param eventMatcher for matching PropertyChangeEvents that will be delivered to the
+     *        ObservableElementList
+     * @return an ObservableElementList.Connector for the specified class
+     */
+    public static <E> ObservableElementList.Connector<E> beanConnector(Class<E> beanClass,
+            Matcher<PropertyChangeEvent> eventMatcher) {
+        return new BeanConnector<E>(beanClass, eventMatcher);
     }
 
     /**
@@ -420,7 +540,42 @@ public final class GlazedLists {
      * @return an ObservableElementList.Connector for the specified class
      */
     public static <E> ObservableElementList.Connector<E> beanConnector(Class<E> beanClass, String addListener, String removeListener) {
-        return new JavaBeanEventListConnector<E>(beanClass, addListener, removeListener);
+        return new BeanConnector<E>(beanClass, addListener, removeListener);
+    }
+    
+    /**
+     * Create a new Connector for the {@link ObservableElementList} that works with
+     * JavaBeans' {@link java.beans.PropertyChangeListener}. The methods to add
+     * and remove listeners are specified by name. Such methods must take a single
+     * {@link java.beans.PropertyChangeListener} argument.
+     *
+     * <p> The event matcher allows filtering of {@link java.beans.PropertyChangeEvent}s. 
+     * Only matching events are delivered to the ObservableElementList.
+     * To create a matcher that matches PropertyChangeEvents by property names, you can use
+     * {@link Matchers#propertyEventNameMatcher(boolean, String[])}
+     * 
+     * @param beanClass a class with both methods as specified.
+     * @param addListener a method name such as "addPropertyChangeListener"
+     * @param removeListener a method name such as "removePropertyChangeListener"
+     * @param eventMatcher for matching PropertyChangeEvents that will be delivered to the
+     *        ObservableElementList
+     * @return an ObservableElementList.Connector for the specified class
+     */
+    public static <E> ObservableElementList.Connector<E> beanConnector(Class<E> beanClass,
+            String addListener, String removeListener, Matcher<PropertyChangeEvent> eventMatcher) {
+        return new BeanConnector<E>(beanClass, addListener, removeListener, eventMatcher);
+    }
+
+    /**
+     * Create a new Connector for the {@link ObservableElementList} that works
+     * with subclasses of the archaic {@link Observable} base class. Each
+     * element of the ObservableElementList <strong>must</strong> extend the
+     * Observable base class.
+     *
+     * @return an ObservableElementList.Connector for objects that extend {@link Observable}
+     */
+    public static <E extends Observable> ObservableElementList.Connector<E> observableConnector() {
+        return new ObservableConnector<E>();
     }
 
     // Matchers // // // // // // // // // // // // // // // // // // // // //
@@ -453,6 +608,23 @@ public final class GlazedLists {
 
     // Functions // // // // // // // // // // // // // // // // // // // // //
 
+    /**
+     * Get a {@link FunctionList.Function} that always returns the given
+     * <code>value</code>, regardless of its input.
+     */
+    public static <E,V> FunctionList.Function<E,V> constantFunction(V value) {
+        return new ConstantFunction<E,V>(value);
+    }
+
+    /**
+     * Get a {@link FunctionList.Function} that extracts the property with the
+     * given <code>propertyName</code> from objects of the given
+     * <code>beanClass</code> and then formats the return value as a String.
+     */
+    public static <E> FunctionList.Function<E,String> toStringFunction(Class<E> beanClass, String propertyName) {
+        return new StringBeanFunction<E>(beanClass, propertyName);
+    }
+
     /**
      * Get a {@link FunctionList.Function} that extracts the property with the
      * given <code>propertyName</code> from objects of the given
@@ -539,12 +711,18 @@ public final class GlazedLists {
      *
      * <p>This can be useful when it is known that an <code>EventList</code>
      * will experience very few mutations compared to read operation and wants
-     * provide a data structure that guarantees fast O(1) reads.
+     * to provide a data structure that guarantees fast O(1) reads.
      *
      * <p>The keys of the MultiMap are determined by evaluating each
-     * <code>source</code> element with the <code>keyMaker</code> function. If
-     * two distinct values, say <code>v1</code> and <code>v2</code> each
-     * produce the key <code>k</code> when they are evaluated by the
+     * <code>source</code> element with the <code>keyMaker</code> function.
+     * This form of the MultiMap requires that the keys produced by the
+     * <code>keyMaker</code> are {@link Comparable} and that the natural
+     * ordering of those keys also defines the grouping of values. If either
+     * of those assumptions are false, consider using
+     * {@link #syncEventListToMultiMap(EventList, FunctionList.Function, Comparator)}.
+     *
+     * <p>If two distinct values, say <code>v1</code> and <code>v2</code> each
+     * produce a common key, <code>k</code>, when they are evaluated by the
      * <code>keyMaker</code> function, then a corresponding entry in the
      * MultiMap will resemble:
      *
@@ -556,7 +734,7 @@ public final class GlazedLists {
      *
      * <p><code>{"Andy", "Arthur", "Jesse", "Holger", "James"}</code>
      *
-     * <p>The MultMap returned by this method would thus resemble:
+     * <p>The MultiMap returned by this method would thus resemble:
      *
      * <p><code>
      * "A" -> {"Andy", "Arthur"}<br>
@@ -573,6 +751,8 @@ public final class GlazedLists {
      *   <li>the mutating methods of {@link Map#values()} and its {@link Iterator}
      *   <li>the mutating methods of {@link Map#entrySet()} and its {@link Iterator}
      *   <li>the {@link Map.Entry#setValue} method
+     *   <li>the mutating methods of {@link Map} itself, including {@link Map#put},
+     *       {@link Map#putAll}, {@link Map#remove}, and {@link Map#clear}
      * </ul>
      *
      * For information on MultiMaps go <a href="http://en.wikipedia.org/wiki/Multimap"/>here</a>.
@@ -581,11 +761,136 @@ public final class GlazedLists {
      *      Each change to this {@link EventList} will be applied to the
      *      MultiMap
      * @param keyMaker the {@link FunctionList.Function} which produces a key
-     *      for each value in the <code>source</code>
+     *      for each value in the <code>source</code>. It is imperative that the
+     *      keyMaker produce <strong>immutable</strong> objects.
      * @return a MultiMap which remains in sync with changes that occur to the
      *      underlying <code>source</code> {@link EventList}
      */
-    public static <K, V> Map<Comparable<K>, List<V>> syncEventListToMultiMap(EventList<V> source, FunctionList.Function<V, ? extends Comparable<K>> keyMaker) {
-        return new GroupingListMultiMap<K, V>(source, keyMaker);
+    public static <K extends Comparable, V> DisposableMap<K, List<V>> syncEventListToMultiMap(EventList<V> source, FunctionList.Function<V, ? extends K> keyMaker) {
+        return syncEventListToMultiMap(source, keyMaker, comparableComparator());
+    }
+
+    /**
+     * Synchronize the specified {@link EventList} to a MultiMap that is
+     * returned from this method. Each time the {@link EventList} is changed
+     * the MultiMap is updated to reflect the change.
+     *
+     * <p>This can be useful when it is known that an <code>EventList</code>
+     * will experience very few mutations compared to read operation and wants
+     * to provide a data structure that guarantees fast O(1) reads.
+     *
+     * <p>The keys of the MultiMap are determined by evaluating each
+     * <code>source</code> element with the <code>keyMaker</code> function.
+     * This form of the MultiMap makes no assumptions about the keys of the
+     * MultiMap and relies on the given <code>keyGrouper</code> to define the
+     * grouping of values.
+     *
+     * <p>If two distinct values, say <code>v1</code> and <code>v2</code> each
+     * produce a common key, <code>k</code>, when they are evaluated by the
+     * <code>keyMaker</code> function, then a corresponding entry in the
+     * MultiMap will resemble:
+     *
+     * <p><code>k -> {v1, v2}</code>
+     *
+     * <p>For example, assume the <code>keyMaker</code> function returns the
+     * first letter of a name and the <code>source</code> {@link EventList}
+     * contains the names:
+     *
+     * <p><code>{"Andy", "Arthur", "Jesse", "Holger", "James"}</code>
+     *
+     * <p>The MultiMap returned by this method would thus resemble:
+     *
+     * <p><code>
+     * "A" -> {"Andy", "Arthur"}<br>
+     * "H" -> {"Holger"}<br>
+     * "J" -> {"Jesse", "James"}<br>
+     * </code>
+     *
+     * <p>It is important to note that all mutating methods on the {@link Map}
+     * interface "write through" to the backing {@link EventList} as expected.
+     * These mutating methods include:
+     *
+     * <ul>
+     *   <li>the mutating methods of {@link Map#keySet()} and its {@link Iterator}
+     *   <li>the mutating methods of {@link Map#values()} and its {@link Iterator}
+     *   <li>the mutating methods of {@link Map#entrySet()} and its {@link Iterator}
+     *   <li>the {@link Map.Entry#setValue} method
+     *   <li>the mutating methods of {@link Map} itself, including {@link Map#put},
+     *       {@link Map#putAll}, {@link Map#remove}, and {@link Map#clear}
+     * </ul>
+     *
+     * For information on MultiMaps go <a href="http://en.wikipedia.org/wiki/Multimap"/>here</a>.
+     *
+     * @param source the {@link EventList} which provides the master view.
+     *      Each change to this {@link EventList} will be applied to the
+     *      MultiMap
+     * @param keyMaker the {@link FunctionList.Function} which produces a key
+     *      for each value in the <code>source</code>. It is imperative that the
+     *      keyMaker produce <strong>immutable</strong> objects.
+     * @param keyGrouper the {@link Comparator} which groups together values
+     *      that share common keys
+     * @return a MultiMap which remains in sync with changes that occur to the
+     *      underlying <code>source</code> {@link EventList}
+     */
+    public static <K, V> DisposableMap<K, List<V>> syncEventListToMultiMap(EventList<V> source, FunctionList.Function<V, ? extends K> keyMaker, Comparator<? super K> keyGrouper) {
+        return new GroupingListMultiMap<K, V>(source, keyMaker, keyGrouper);
+    }
+
+    /**
+     * Synchronize the specified {@link EventList} to a Map that is returned
+     * from this method. Each time the {@link EventList} is changed the Map is
+     * updated to reflect the change.
+     *
+     * <p>This can be useful when it is known that an <code>EventList</code>
+     * will experience very few mutations compared to read operation and wants
+     * to provide a data structure that guarantees fast O(1) reads.
+     *
+     * <p>The keys of the Map are determined by evaluating each
+     * <code>source</code> element with the <code>keyMaker</code> function.
+     * The Map implementation assumes that each value has a unique key, and
+     * verifies this invariant at runtime, throwing a RuntimeException if it
+     * is ever violated.
+     *
+     * For example, if two distinct values, say <code>v1</code> and
+     * <code>v2</code> each produce the key <code>k</code> when they are
+     * evaluated by the <code>keyMaker</code> function, an
+     * {@link IllegalStateException} is thrown to proactively indicate the
+     * error.
+     *
+     * <p>As for example of normal usage, assume the <code>keyMaker</code>
+     * function returns the first letter of a name and the <code>source</code>
+     * {@link EventList} contains the names:
+     *
+     * <p><code>{"Kevin", "Jesse", "Holger"}</code>
+     *
+     * <p>The Map returned by this method would thus resemble:
+     *
+     * <p><code>
+     * "K" -> "Kevin"<br>
+     * "J" -> "Jesse"<br>
+     * "H" -> "Holger"<br>
+     * </code>
+     *
+     * <p>It is important to note that all mutating methods on the {@link Map}
+     * interface "write through" to the backing {@link EventList} as expected.
+     * These mutating methods include:
+     *
+     * <ul>
+     *   <li>the mutating methods of {@link Map#keySet()} and its {@link Iterator}
+     *   <li>the mutating methods of {@link Map#values()} and its {@link Iterator}
+     *   <li>the mutating methods of {@link Map#entrySet()} and its {@link Iterator}
+     *   <li>the {@link Map.Entry#setValue} method
+     * </ul>
+     *
+     * @param source the {@link EventList} which provides the values of the map.
+     *      Each change to this {@link EventList} will be applied to the Map.
+     * @param keyMaker the {@link FunctionList.Function} which produces a key
+     *      for each value in the <code>source</code>. It is imperative that the
+     *      keyMaker produce <strong>immutable</strong> objects.
+     * @return a Map which remains in sync with changes that occur to the
+     *      underlying <code>source</code> {@link EventList}
+     */
+    public static <K, V> DisposableMap<K, V> syncEventListToMap(EventList<V> source, FunctionList.Function<V, K> keyMaker) {
+        return new FunctionListMap<K, V>(source, keyMaker);
     }
 }
\ No newline at end of file
