@@ -7,10 +7,9 @@ import ca.odell.glazedlists.event.ListEvent;
 import ca.odell.glazedlists.impl.adt.Barcode;
 import ca.odell.glazedlists.impl.adt.BarcodeIterator;
 
-import java.util.List;
 import java.util.ArrayList;
 import java.util.EventListener;
-import java.util.Iterator;
+import java.util.List;
 
 /**
  * A list that fires update events whenever elements are modified in place.
@@ -21,6 +20,10 @@ import java.util.Iterator;
  * which contains the necessary logic for registering and unregistering a
  * listener capable of detecting modifications to an observable list element.
  *
+ * <p><strong><font color="#FF0000">Warning:</font></strong> This class is
+ * thread ready but not thread safe. See {@link EventList} for an example
+ * of thread safe code.
+ *
  * <p><table border="1" width="100%" cellpadding="3" cellspacing="0">
  * <tr class="TableHeadingColor"><td colspan=2><font size="+2"><b>EventList Overview</b></font></td></tr>
  * <tr><td class="TableSubHeadingColor"><b>Writable:</b></td><td>yes</td></tr>
@@ -45,6 +48,7 @@ public class ObservableElementList<E> extends TransformedList<E, E> {
      * elements since list removals broadcast ListEvents which do not include
      * the removed element as part of the ListEvent. We use this list to locate
      * removed elements for the purpose of unregistering listeners from them.
+     * todo remove this list when ListEvent can reliably furnish us with a deleted value
      */
     private List<E> observedElements;
 
@@ -55,7 +59,7 @@ public class ObservableElementList<E> extends TransformedList<E, E> {
      * listener is responsible for calling {@link #elementChanged(Object)}
      * to notify this list of the changed object.
      */
-    private Connector<E> elementConnector = null;
+    private Connector<? super E> elementConnector = null;
 
     /**
      * <tt>true</tt> indicates a single shared EventListener is used for each
@@ -102,7 +106,7 @@ public class ObservableElementList<E> extends TransformedList<E, E> {
      *      this list to the given <code>elementConnector</code> by calling
      *      {@link Connector#setObservableElementList(ObservableElementList)}.
      */
-    public ObservableElementList(EventList<E> source, Connector<E> elementConnector) {
+    public ObservableElementList(EventList<E> source, Connector<? super E> elementConnector) {
         super(source);
 
         this.elementConnector = elementConnector;
@@ -120,13 +124,12 @@ public class ObservableElementList<E> extends TransformedList<E, E> {
         this.singleEventListenerRegistry.addWhite(0, source.size());
 
         // add listeners to all source list elements
-        int index = 0;
-        for (Iterator<E> iter = this.iterator(); iter.hasNext(); index++) {
+        for (int i = 0, n = size(); i < n; i++) {
             // connect a listener to the element
-            final EventListener listener = this.connectElement(iter.next());
+            final EventListener listener = this.connectElement(get(i));
 
             // record the listener in the registry
-            this.registerListener(index, listener, false);
+            this.registerListener(i, listener, false);
         }
 
         // begin listening to the source list
@@ -154,7 +157,13 @@ public class ObservableElementList<E> extends TransformedList<E, E> {
 
             // unregister a listener on the deleted object
             } else if (changeType == ListEvent.DELETE) {
-                final E deleted = this.observedElements.remove(changeIndex);
+                // try to get the previous value through the ListEvent
+                E deleted = listChanges.getOldValue();
+                E deletedElementFromPrivateCopy = this.observedElements.remove(changeIndex);
+
+                // if the ListEvent could give us the previous value, use the value from our private copy of the source
+                if (deleted == ListEvent.UNKNOWN_VALUE)
+                    deleted = deletedElementFromPrivateCopy;
 
                 // remove the listener from the registry
                 final EventListener listener = this.unregisterListener(changeIndex);
@@ -163,7 +172,12 @@ public class ObservableElementList<E> extends TransformedList<E, E> {
 
             // register/unregister listeners if the value at the changeIndex is now a different object
             } else if (changeType == ListEvent.UPDATE) {
-                final E previousValue = this.observedElements.get(changeIndex);
+                E previousValue = listChanges.getOldValue();
+
+                // if the ListEvent could give us the previous value, use the value from our private copy of the source
+                if (previousValue == ListEvent.UNKNOWN_VALUE)
+                    previousValue = this.observedElements.get(changeIndex);
+
                 final E newValue = get(changeIndex);
 
                 // if a different object is present at the index
@@ -311,18 +325,20 @@ public class ObservableElementList<E> extends TransformedList<E, E> {
     private void switchToMultiListenerMode() {
         if (!this.singleListenerMode) throw new IllegalStateException();
 
-        // build a new data structure appropriate for individual storing
+        // build a new data structure appropriate for storing individual
         // listeners for each observed element
         this.multiEventListenerRegistry = new ArrayList<EventListener>(this.source.size());
+        for (int i = 0; i < source.size(); i++)
+            this.multiEventListenerRegistry.add(null);
 
         // for each black entry in the singleEventListenerRegistry create an
         // entry in the multiEventListenerRegistry at the corresponding index
         // for the singleEventListener
         for (BarcodeIterator iter = this.singleEventListenerRegistry.iterator(); iter.hasNextBlack();) {
             iter.nextBlack();
-            this.multiEventListenerRegistry.add(iter.getIndex(), this.singleEventListener);
+            this.multiEventListenerRegistry.set(iter.getIndex(), this.singleEventListener);
         }
-
+        
         // null out the reference to the single EventListener,
         // since we'll now track the EventListener for each element
         this.singleEventListener = null;
@@ -376,7 +392,7 @@ public class ObservableElementList<E> extends TransformedList<E, E> {
     }
 
     /**
-     * Handle a listener being fired for the specified <code>listElement</code>.
+     * Handle a listener being notified for the specified <code>listElement</code>.
      * This method causes a ListEvent to be fired from this EventList indicating
      * an update occurred at all locations of the given <code>listElement</code>.
      *
@@ -402,7 +418,7 @@ public class ObservableElementList<E> extends TransformedList<E, E> {
             // locate all indexes containing the given listElement
             for (int i = 0, n = size(); i < n; i++) {
                 if (listElement == get(i))
-                    this.updates.addUpdate(i);
+                    this.updates.elementUpdated(i, listElement);
             }
 
             this.updates.commitEvent();
@@ -454,6 +470,6 @@ public class ObservableElementList<E> extends TransformedList<E, E> {
          * @param list the ObservableElementList containing the elements to
          *      observe
          */
-        public void setObservableElementList(ObservableElementList<E> list);
+        public void setObservableElementList(ObservableElementList<? extends E> list);
     }
 }
\ No newline at end of file
