@@ -3,24 +3,39 @@
 /*                                                     O'Dell Engineering Ltd.*/
 package ca.odell.glazedlists;
 
-import ca.odell.glazedlists.impl.Grouper;
-import ca.odell.glazedlists.impl.adt.*;
-import ca.odell.glazedlists.impl.adt.barcode2.SimpleTree;
-import ca.odell.glazedlists.impl.adt.barcode2.Element;
-import ca.odell.glazedlists.impl.adt.barcode2.SimpleTreeIterator;
 import ca.odell.glazedlists.event.ListEvent;
+import ca.odell.glazedlists.impl.Grouper;
+import ca.odell.glazedlists.impl.adt.Barcode;
+import ca.odell.glazedlists.impl.adt.BarcodeIterator;
+import ca.odell.glazedlists.impl.adt.barcode2.Element;
+import ca.odell.glazedlists.impl.adt.barcode2.SimpleTree;
+import ca.odell.glazedlists.impl.adt.barcode2.SimpleTreeIterator;
 
+import java.util.Collections;
 import java.util.Comparator;
 import java.util.List;
-import java.util.Collections;
 
 /**
  * A list that adds separator objects before each group of elements.
  *
+ * <p>SeparatorList is writable, however, attempts to write over separators
+ * will always produce an {@link IllegalArgumentException}. For example,
+ * calling {@link #add(int, Object)}, {@link #set(int, Object)} or
+ * {@link #remove(int)} with a an index that actually corresponds to a
+ * separator in this list will produce an {@link IllegalArgumentException}.
+ * This is because there is no corresponding index for separators in the source
+ * list; separators are added by this SeparatorList. All index-based write
+ * operations must be performed on indexes known to correspond to non-separator
+ * elements.
+ *
  * <p><strong>Warning:</strong> this class won't work very well with generics
  * because separators are mixed in, which will be a different class than the
  * other list elements.
  *
+ * <p><strong><font color="#FF0000">Warning:</font></strong> This class is
+ * thread ready but not thread safe. See {@link EventList} for an example
+ * of thread safe code.
+ *
  * <p><strong>Developer Preview</strong> this class is still under heavy development
  * and subject to API changes. It's also really slow at the moment and won't scale
  * to lists of size larger than a hundred or so efficiently.
@@ -35,17 +50,30 @@ public class SeparatorList<E> extends TransformedList<E, E> {
     private static final Object SOURCE_ELEMENT = Barcode.WHITE;
 
     /** how many elements before we get a separator, such as 1 or 2 */
-    private int minimumSizeForSeparator;
+    private final int minimumSizeForSeparator;
 
     /** manage collapsed elements */
     private Barcode collapsedElements;
 
-
     /**
-     * Create a {@link SeparatorList}...
+     * Construct a SeparatorList overtop of the <code>source</code> list by
+     * using the given <code>comparator</code> to compute groups of similar
+     * source items. For each group a single separator will be present in this
+     * SeparatorList provided the group contains at least the
+     * <code>minimumSizeForSeparator</code> number of items (otherwise they are
+     * left without a separator). In addition this SeparatorList will never
+     * show more than the <code>defaultLimit</code> number of group elements
+     * from any given group.
+     *
+     * @param source the list containing the raw items to be grouped
+     * @param comparator the Comparator which defines the grouping logic
+     * @param minimumSizeForSeparator the number of elements which must exist
+     *      in a group in order for a separator to be created
+     * @param defaultLimit the maximum number of element to display for a group;
+     *      extra elements are truncated
      */
-    public SeparatorList(EventList<E> source, Comparator<E> comparator, int minimumSizeForSeparator, int defaultLimit) {
-        super(new SeparatorInjectorList<E>(new SortedList<E>(source, comparator), comparator, defaultLimit));
+    public SeparatorList(EventList<E> source, Comparator<? super E> comparator, int minimumSizeForSeparator, int defaultLimit) {
+        super(new SeparatorInjectorList<E>(new SortedList<E>(source, comparator), defaultLimit));
         this.separatorSource = (SeparatorInjectorList<E>)super.source;
         this.minimumSizeForSeparator = minimumSizeForSeparator;
 
@@ -78,6 +106,11 @@ public class SeparatorList<E> extends TransformedList<E, E> {
         return collapsedElements.getIndex(mutationIndex, Barcode.BLACK);
     }
 
+    /** {@inheritDoc} */
+    protected boolean isWritable() {
+        return true;
+    }
+
     /**
      * Set the {@link Comparator} used to determine how elements are split
      * into groups.
@@ -89,14 +122,18 @@ public class SeparatorList<E> extends TransformedList<E, E> {
      * of thread safe code.
      */
     public void setComparator(Comparator<E> comparator) {
-        // this implementation loses selection, but that's the best we can do
-        // with the current limitations of the Glazed Lists ListEventAssembler.
-        // What we really need here is the ability to fire an event that contains
-        // both reordering and structure change information.
-        updates.beginEvent(false);
+        final boolean isEmpty = isEmpty();
 
-        // remove all
-        updates.addDelete(0, size() - 1);
+        if (!isEmpty) {
+            // this implementation loses selection, but that's the best we can do
+            // with the current limitations of the Glazed Lists ListEventAssembler.
+            // What we really need here is the ability to fire an event that contains
+            // both reordering and structure change information.
+            updates.beginEvent();
+
+            // remove all
+            updates.addDelete(0, size() - 1);
+        }
 
         // make the change to the sorted source, the grouper will respond but
         // the {@link SeparatorInjectorList} doesn't fire any events forward when
@@ -104,12 +141,14 @@ public class SeparatorList<E> extends TransformedList<E, E> {
         SortedList<E> sortedList = (SortedList<E>)separatorSource.source;
         sortedList.setComparator(comparator);
 
-        // rebuild which elements are collapsed out
-        rebuildCollapsedElements();
+        if (!isEmpty) {
+            // rebuild which elements are collapsed out
+            rebuildCollapsedElements();
 
-        // insert all again
-        updates.addInsert(0, size() - 1);
-        updates.commitEvent();
+            // insert all again
+            updates.addInsert(0, size() - 1);
+            updates.commitEvent();
+        }
     }
 
     /**
@@ -386,7 +425,7 @@ public class SeparatorList<E> extends TransformedList<E, E> {
      * as a separate transformation from the hiding of separators
      * and collapsed elements.
      */
-    private static class SeparatorInjectorList<E> extends TransformedList<E, E> {
+    static class SeparatorInjectorList<E> extends TransformedList<E, E> {
 
         /** the grouping service manages finding where to insert groups */
         private final Grouper<E> grouper;
@@ -425,17 +464,21 @@ public class SeparatorList<E> extends TransformedList<E, E> {
         /** a list of {@link Separator}s, one for each separator in the list */
         private SimpleTree<GroupSeparator> separators;
 
-        /** the number of elements to show in each group, such as 0, 5, or {@link Integer.MAX_VALUE} */
+        /** the number of elements to show in each group, such as 0, 5, or {@link Integer#MAX_VALUE} */
         private int defaultLimit;
 
         /**
-         * Create a new {@link UniqueList} that determines groups using the specified
-         * {@link Comparator}. Elements that the {@link Comparator} determines are
-         * equal will share a common separator.
+         * Create a new {@link SeparatorInjectorList} that groups together like
+         * items using the Comparator from the <code>source</code>. Elements
+         * that the {@link Comparator} determines are equal will share a common
+         * separator.
          *
-         * @see GlazedLists#beanPropertyComparator
+         * @param source the sorted list of items with a Comparator that defines
+         *      the group boundaries
+         * @param defaultLimit the maximum number of items to include in a
+         *      group; all remaining items will be truncated
          */
-        public SeparatorInjectorList(SortedList<E> source, Comparator<E> comparator, int defaultLimit) {
+        public SeparatorInjectorList(SortedList<E> source, int defaultLimit) {
             super(source);
             this.defaultLimit = defaultLimit;
 
@@ -486,7 +529,7 @@ public class SeparatorList<E> extends TransformedList<E, E> {
         /** {@inheritDoc} */
         protected int getSourceIndex(int mutationIndex) {
             Object type = insertedSeparators.get(mutationIndex);
-            if(type == SEPARATOR) return -1;
+            if(type == SEPARATOR) throw new IllegalArgumentException("No source index exists for the separator located at index " + mutationIndex);
             else if(type == SOURCE_ELEMENT) return insertedSeparators.getColourIndex(mutationIndex, SOURCE_ELEMENT);
             else throw new IllegalStateException();
         }
@@ -497,6 +540,10 @@ public class SeparatorList<E> extends TransformedList<E, E> {
             else throw new IllegalStateException();
         }
 
+        /** {@inheritDoc} */
+        protected boolean isWritable() {
+            return true;
+        }
 
         /** {@inheritDoc} */
         public int size() {
@@ -568,8 +615,9 @@ public class SeparatorList<E> extends TransformedList<E, E> {
          * Fire two events, one for the group (the separator) and another for the
          * actual list element.
          */
-        private class GrouperClient implements Grouper.Client {
-            public void groupChanged(int index, int groupIndex, int groupChangeType, boolean primary, int elementChangeType) {
+        private class GrouperClient implements Grouper.Client<E> {
+            public void groupChanged(int index, int groupIndex, int groupChangeType, boolean primary, int elementChangeType, E oldValue, E newValue) {
+                boolean fixSeparatorForInsertGroupUpdateElement = false;
                 // handle the group change first
                 if(groupChangeType == ListEvent.INSERT) {
                     int expandedIndex = index + groupIndex;
@@ -601,6 +649,22 @@ public class SeparatorList<E> extends TransformedList<E, E> {
                     updates.addInsert(expandedIndex);
                 } else if(elementChangeType == ListEvent.UPDATE) {
                     int expandedIndex = index + groupIndex + 1;
+                    // if we inserted a separator directly before an existing separator,
+                    // we must increase update index for element by one
+                    if (groupChangeType == ListEvent.INSERT) {
+                        int separatorCount = insertedSeparators.colourSize(SEPARATOR);
+                        if (groupIndex + 1 < separatorCount) {
+                            // separator at groupIndex is not the last one...
+                            int nextSeparatorsIndex = insertedSeparators.getIndex(groupIndex + 1, SEPARATOR);
+                            if (nextSeparatorsIndex == expandedIndex) {
+                                // separator at groupIndex + 1 is located at the element update position,
+                                // so increment update position by one...
+                                expandedIndex++;
+                                // ...and fix location of separator at groupIndex + 1
+                                fixSeparatorForInsertGroupUpdateElement = true;
+                            }
+                        }
+                    }
                     updates.addUpdate(expandedIndex);
                 } else if(elementChangeType == ListEvent.DELETE) {
                     int expandedIndex = index + groupIndex + 1;
@@ -608,6 +672,19 @@ public class SeparatorList<E> extends TransformedList<E, E> {
                     updates.addDelete(expandedIndex);
                 }
 
+                if (fixSeparatorForInsertGroupUpdateElement) {
+                    // fix special case:
+                    // the location of separator at groupIndex + 1 must be increased by one
+
+                    int wrongSeparatorIndex = index + groupIndex + 1;
+                    assert wrongSeparatorIndex == insertedSeparators.getIndex(groupIndex + 1, SEPARATOR);
+
+                    insertedSeparators.remove(wrongSeparatorIndex, 1);
+                    updates.addDelete(wrongSeparatorIndex);
+                    insertedSeparators.add(wrongSeparatorIndex + 1, SEPARATOR, 1);
+                    updates.addInsert(wrongSeparatorIndex + 1);
+                }
+
                 // Special case out the shift operation. The Grouper automatically
                 // handles shifts, but SeparatorList needs to manage them independently.
                 // Here we respond to the grouper and the separators barcode getting
@@ -616,19 +693,18 @@ public class SeparatorList<E> extends TransformedList<E, E> {
                 // Grouper works to fire a special flag called 'shift' with the
                 // value true whenever the group joined is a RIGHT_GROUP
                 int shiftGroupIndex = groupIndex + 1;
-                if(groupChangeType == ListEvent.DELETE && elementChangeType == ListEvent.DELETE
+                if(groupChangeType == ListEvent.DELETE && elementChangeType != ListEvent.INSERT
                         && shiftGroupIndex < insertedSeparators.colourSize(SEPARATOR)
                         && shiftGroupIndex < grouper.getBarcode().colourSize(Grouper.UNIQUE)) {
                     int collapsedGroupStartIndex = grouper.getBarcode().getIndex(shiftGroupIndex, Grouper.UNIQUE);
                     int separatorsIndex = insertedSeparators.getIndex(shiftGroupIndex , SEPARATOR);
                     //String was = insertedSeparators.toString();
-                    if(collapsedGroupStartIndex + shiftGroupIndex < separatorsIndex) {
+                     if(collapsedGroupStartIndex + shiftGroupIndex < separatorsIndex) {
                         insertedSeparators.remove(separatorsIndex, 1);
                         updates.addDelete(separatorsIndex);
                         insertedSeparators.add(collapsedGroupStartIndex + shiftGroupIndex, SEPARATOR, 1);
                         updates.addInsert(collapsedGroupStartIndex + shiftGroupIndex);
                         //String now = insertedSeparators.toString();
-                        index++;
                         //System.out.println("Changed from " + was + " to " + now);
                     }
                 }
@@ -638,7 +714,7 @@ public class SeparatorList<E> extends TransformedList<E, E> {
         /**
          * Implement the {@link Separator} interface in the most natural way.
          */
-        private class GroupSeparator implements Separator<E> {
+        class GroupSeparator implements Separator<E> {
             private int limit = Integer.MAX_VALUE;
             private int size;
             private E first;
@@ -686,7 +762,7 @@ public class SeparatorList<E> extends TransformedList<E, E> {
             }
 
             /**
-             * Set the {@link IndexedTreeNode} that this {@link Separator} can
+             * Set the {@link Element} that this {@link Separator} can
              * use to find its index in the overall list of {@link Separator}s;
              */
             public void setNode(Element<GroupSeparator> node) {
