@@ -3,11 +3,11 @@
 /*                                                     O'Dell Engineering Ltd.*/
 package ca.odell.glazedlists;
 
+import ca.odell.glazedlists.event.ListEvent;
+import ca.odell.glazedlists.impl.GlazedListsImpl;
 import ca.odell.glazedlists.impl.adt.barcode2.Element;
 import ca.odell.glazedlists.impl.adt.barcode2.SimpleTree;
 import ca.odell.glazedlists.impl.adt.barcode2.SimpleTreeIterator;
-import ca.odell.glazedlists.impl.GlazedListsImpl;
-import ca.odell.glazedlists.event.ListEvent;
 
 import java.util.*;
 
@@ -24,6 +24,10 @@ import java.util.*;
  * breaks the contract required by {@link List}. See {@link EventList}
  * for an example.
  *
+ * <p><strong><font color="#FF0000">Warning:</font></strong> This class is
+ * thread ready but not thread safe. See {@link EventList} for an example
+ * of thread safe code.
+ *
  * <p><table border="1" width="100%" cellpadding="3" cellspacing="0">
  * <tr class="TableHeadingColor"><td colspan=2><font size="+2"><b>EventList Overview</b></font></td></tr>
  * <tr><td class="TableSubHeadingColor"><b>Writable:</b></td><td>yes</td></tr>
@@ -79,13 +83,25 @@ public final class SortedList<E> extends TransformedList<E,E> {
 
     /** one of {@link #STRICT_SORT_ORDER} or {@link #AVOID_MOVING_ELEMENTS}. */
     private int mode = STRICT_SORT_ORDER;
-
+    
+    /**
+     * Creates a {@link SortedList} that sorts the specified {@link EventList}.
+     * All elements in the specified {@link EventList} must implement {@link Comparable}.
+     * 
+     * @param source the {@link EventList} to be sorted
+     */
+    public static <E extends Comparable<? super E>> SortedList<E> create(EventList<E> source) {
+        return new SortedList<E>(source);
+    }
 
     /**
      * Creates a {@link SortedList} that sorts the specified {@link EventList}.
      * Because this constructor takes no {@link Comparator} argument, all
      * elements in the specified {@link EventList} must implement {@link Comparable}
      * or a {@link ClassCastException} will be thrown.
+     * <p>Usage of factory method {@link #create(EventList)} is preferable.
+     * 
+     * @param source the {@link EventList} to be sorted
      */
     public SortedList(EventList<E> source) {
         this(source, (Comparator<E>)GlazedLists.comparableComparator());
@@ -217,6 +233,7 @@ public final class SortedList<E> extends TransformedList<E,E> {
         // first update the offset tree for all changes, and keep the changed nodes in a list
         LinkedList<Element> insertNodes = new LinkedList<Element>();
         List<Element<Element>> updateNodes = new ArrayList<Element<Element>>();
+        List<E> previousValues = new ArrayList<E>();
 
         // Update the indexed tree so it matches the source.
         // Save the nodes to be inserted and updated as well
@@ -237,13 +254,15 @@ public final class SortedList<E> extends TransformedList<E,E> {
                 Element sortedNode = unsortedNode.get();
                 sortedNode.setSorted(Element.PENDING);
                 updateNodes.add(sortedNode);
+                previousValues.add(listChanges.getOldValue());
 
             // on delete, delete the index and sorted node
             } else if(changeType == ListEvent.DELETE) {
                 Element<Element> unsortedNode = unsorted.get(unsortedIndex);
+                E deleted = listChanges.getOldValue();
                 unsorted.remove(unsortedNode);
                 int deleteSortedIndex = deleteByUnsortedNode(unsortedNode);
-                updates.addDelete(deleteSortedIndex);
+                updates.elementDeleted(deleteSortedIndex, deleted);
 
             }
         }
@@ -251,8 +270,8 @@ public final class SortedList<E> extends TransformedList<E,E> {
         // decide which updated elements need to be shifted. We walk through the
         // tree, marking updated elements as sorted or unsorted depending on their
         // value relative to their neighbours
-        for(Iterator<Element<Element>> i = updateNodes.iterator(); i.hasNext(); ) {
-            Element<Element> sortedNode = i.next();
+        for(int i = 0, size = updateNodes.size(); i < size; i++) {
+            Element<Element> sortedNode = updateNodes.get(i);
             // we may have already handled this via a neighbour 
             if(sortedNode.getSorted() != Element.PENDING) continue;
 
@@ -302,24 +321,25 @@ public final class SortedList<E> extends TransformedList<E,E> {
         }
 
         // fire update events
-        for(Iterator<Element<Element>> i = updateNodes.iterator(); i.hasNext(); ) {
-            Element<Element> sortedNode = i.next();
+        for(int i = 0, size = updateNodes.size(); i < size; i++) {
+            E previous = previousValues.get(i);
+            Element<Element> sortedNode = updateNodes.get(i);
             assert(sortedNode.getSorted() != Element.PENDING);
             int originalIndex = sorted.indexOfNode(sortedNode, ALL_COLORS);
 
             // the element is still in sorted order, forward the update event
             if(sortedNode.getSorted() == Element.SORTED) {
-                updates.addUpdate(originalIndex);
+                updates.elementUpdated(originalIndex, previous);
 
             // sort order is not enforced so we lose perfect sorting order
             // but we don't need to move elements around
             } else if(mode == AVOID_MOVING_ELEMENTS) {
-                updates.addUpdate(originalIndex);
+                updates.elementUpdated(originalIndex, previous);
 
             // sort order is enforced so move the element to its new location
             } else {
                 sorted.remove(sortedNode);
-                updates.addDelete(originalIndex);
+                updates.elementDeleted(originalIndex, previous);
                 int insertedIndex = insertByUnsortedNode(sortedNode.get());
                 updates.addInsert(insertedIndex);
             }
@@ -393,7 +413,7 @@ public final class SortedList<E> extends TransformedList<E,E> {
 
     /**
      * Set the {@link Comparator} in use in this {@link EventList}. This will
-     * sort the {@link EventList} into a new order.
+     * sort the source {@link EventList} into a new order.
      *
      * <p>Performance Note: sorting will take <code>O(N * Log N)</code> time.
      *
@@ -413,7 +433,7 @@ public final class SortedList<E> extends TransformedList<E,E> {
         // keep the old trees to construct the reordering
         SimpleTree previousSorted = sorted;
         // create the sorted list with a simple comparator
-        Comparator treeComparator = null;
+        final Comparator treeComparator;
         if(comparator != null) treeComparator = new ElementComparator(comparator);
         else treeComparator = new ElementRawOrderComparator();
         sorted = new SimpleTree<Element>(treeComparator);
@@ -633,17 +653,11 @@ public final class SortedList<E> extends TransformedList<E,E> {
          * Compares the alpha object to the beta object by their indices.
          */
         public int compare(Object alpha, Object beta) {
-            try {
-                Element alphaTreeNode = (Element)alpha;
-                Element betaTreeNode = (Element)beta;
-                int alphaIndex = unsorted.indexOfNode(alphaTreeNode, ALL_COLORS);
-                int betaIndex = unsorted.indexOfNode(betaTreeNode, ALL_COLORS);
-                return alphaIndex - betaIndex;
-            } catch(ClassCastException e) {
-                System.out.println(alpha.getClass());
-                System.out.println(beta.getClass());
-                throw e;
-            }
+            Element alphaTreeNode = (Element)alpha;
+            Element betaTreeNode = (Element)beta;
+            int alphaIndex = unsorted.indexOfNode(alphaTreeNode, ALL_COLORS);
+            int betaIndex = unsorted.indexOfNode(betaTreeNode, ALL_COLORS);
+            return alphaIndex - betaIndex;
         }
     }
 
