@@ -4,8 +4,8 @@
 package ca.odell.glazedlists.impl;
 
 import ca.odell.glazedlists.*;
-import ca.odell.glazedlists.event.ListEventListener;
 import ca.odell.glazedlists.event.ListEvent;
+import ca.odell.glazedlists.event.ListEventListener;
 
 import java.util.*;
 
@@ -16,7 +16,7 @@ import java.util.*;
  * The values of the map are the lists of values from the {@link EventList}
  * which all map to a common key.
  *
- * <p>For example, a list containing
+ * <p>For example, an {@link EventList} containing
  *
  * <pre>
  * {Cherry, Plum, Cranberry, Pineapple, Banana, Prune}
@@ -33,28 +33,28 @@ import java.util.*;
  *
  * @author James Lemieux
  */
-public class GroupingListMultiMap<K, V> implements Map<Comparable<K>, List<V>>, ListEventListener<List<V>> {
+public class GroupingListMultiMap<K, V> implements DisposableMap<K, List<V>>, ListEventListener<List<V>> {
 
     /** The raw values of this Map in an {@link EventList}. */
     private final GroupingList<V> groupingList;
 
-    /** The polished values of this Map in an {@link EventList}. */
+    /** The values of this Map in an {@link EventList}. */
     private final FunctionList<List<V>, List<V>> valueList;
 
     /** The keys of this Map (used to remove entries from the {@link #delegate}) */
-    private final List<Comparable<K>> keyList;
+    private final List<K> keyList;
 
-    /** The keys of this Map made to look like a Set (it is build lazily in {@link #keySet()}) */
-    private Set<Comparable<K>> keySet;
+    /** The keys of this Map made to look like a Set (it is built lazily in {@link #keySet()}) */
+    private Set<K> keySet;
 
     /** The function which produces keys for this multimap. */
-    private final FunctionList.Function<V, ? extends Comparable<K>> keyFunction;
+    private final FunctionList.Function<V, ? extends K> keyFunction;
 
     /** The delegate Map which is kept in synch with {@link #groupingList} changes. */
-    private final Map<Comparable<K>, List<V>> delegate;
+    private final Map<K, List<V>> delegate;
 
-    /** The set of Map.Entry objects in this Map (it is build lazily in {@link #entrySet()}) */
-    private Set<Map.Entry<Comparable<K>, List<V>>> entrySet;
+    /** The set of Map.Entry objects in this Map (it is built lazily in {@link #entrySet()}) */
+    private Set<Map.Entry<K, List<V>>> entrySet;
 
     /**
      * Construct a multimap which maps the keys produced by the
@@ -63,15 +63,20 @@ public class GroupingListMultiMap<K, V> implements Map<Comparable<K>, List<V>>,
      *
      * @param source the raw data which has not yet been grouped
      * @param keyFunction the function capable of producing the keys of this
-     *      {@link Map}; the keys themselves are {@link Comparable} and thus
-     *      also determine the content of the {@link List}s which are the
-     *      values of this {@link Map}.
+     *      {@link Map} from each value
+     * @param keyGrouper the comparator that groups together values which
+     *      have the same key according to the given <code>keyFunction</code>
      */
-    public GroupingListMultiMap(EventList<V> source, FunctionList.Function<V, ? extends Comparable<K>> keyFunction) {
+    public GroupingListMultiMap(EventList<V> source, FunctionList.Function<V, ? extends K> keyFunction, Comparator<? super K> keyGrouper) {
+        if (keyFunction == null)
+            throw new IllegalArgumentException("keyFunction may not be null");
+        if (keyGrouper == null)
+            throw new IllegalArgumentException("keyGrouper may not be null");
+
         this.keyFunction = keyFunction;
 
         // construct a GroupingList which groups together the source elements for common keys
-        this.groupingList = new GroupingList<V>(source, new FunctionComparator(keyFunction));
+        this.groupingList = new GroupingList<V>(source, new FunctionComparator(keyFunction, keyGrouper));
 
         // wrap each List in the GroupingList in a layer that enforces the keyFunction constraints for writes
         this.valueList = new FunctionList<List<V>, List<V>>(this.groupingList, new ValueListFunction());
@@ -79,70 +84,83 @@ public class GroupingListMultiMap<K, V> implements Map<Comparable<K>, List<V>>,
 
         // it is important that the keyList is a BasicEventList since we use its ListIterator, which remains
         // consistent with changes to its underlying data (any other Iterator would throw a ConcurrentModificationException)
-        this.keyList = new BasicEventList<Comparable<K>>(this.groupingList.size());
-        this.delegate = new HashMap<Comparable<K>,List<V>>(this.groupingList.size());
+        this.keyList = new BasicEventList<K>(this.groupingList.size());
+        this.delegate = new HashMap<K,List<V>>(this.groupingList.size());
 
         // initialize both the keyList and the delegate Map
         for (Iterator<List<V>> i = this.valueList.iterator(); i.hasNext();) {
             final List<V> value = i.next();
-            final Comparable key = key(value);
+            final K key = key(value);
             this.keyList.add(key);
             this.delegate.put(key, value);
         }
     }
 
+    /** {@inheritDoc} */
+    public void dispose() {
+        valueList.removeListEventListener(this);
+        valueList.dispose();
+        groupingList.dispose();
+
+        keySet = null;
+        entrySet = null;
+        keyList.clear();
+        delegate.clear();
+    }
+
     /** {@inheritDoc} */
     public int size() {
-        return this.delegate.size();
+        return delegate.size();
     }
 
     /** {@inheritDoc} */
     public boolean isEmpty() {
-        return this.delegate.isEmpty();
+        return delegate.isEmpty();
     }
 
     /** {@inheritDoc} */
     public boolean containsKey(Object key) {
-        return this.delegate.containsKey(key);
+        return delegate.containsKey(key);
     }
 
     /** {@inheritDoc} */
     public boolean containsValue(Object value) {
-        return this.delegate.containsValue(value);
+        return delegate.containsValue(value);
     }
 
     /** {@inheritDoc} */
     public List<V> get(Object key) {
-        return this.delegate.get(key);
+        return delegate.get(key);
     }
 
     /** {@inheritDoc} */
-    public List<V> put(Comparable<K> key, List<V> value) {
-        this.checkKeyValueAgreement(key, value);
+    public List<V> put(K key, List<V> value) {
+        checkKeyValueAgreement(key, value);
 
-        final List<V> removed = (List<V>) this.remove(key);
-        this.groupingList.add(value);
+        final List<V> removed = (List<V>) remove(key);
+        groupingList.add(value);
 
         return removed;
     }
 
     /** {@inheritDoc} */
-    public void putAll(Map<? extends Comparable<K>, ? extends List<V>> m) {
+
+    public void putAll(Map<? extends K, ? extends List<V>> m) {
         // verify the contents of the given Map and ensure all key/value pairs agree with the keyFunction
-        for (Iterator<? extends Entry<? extends Comparable<K>, ? extends List<V>>> i = m.entrySet().iterator(); i.hasNext();) {
-            final Entry<? extends Comparable<K>, ? extends List<V>> entry = i.next();
-            final Comparable<K> key = entry.getKey();
+        for (Iterator<? extends Entry<? extends K, ? extends List<V>>> i = m.entrySet().iterator(); i.hasNext();) {
+            final Entry<? extends K, ? extends List<V>> entry = i.next();
+            final K key = entry.getKey();
             final List<V> value = entry.getValue();
 
-            this.checkKeyValueAgreement(key, value);
+            checkKeyValueAgreement(key, value);
         }
 
         // remove all values currently associated with the keys
-        for (Iterator<? extends Comparable<K>> i = m.keySet().iterator(); i.hasNext();)
-            this.remove(i.next());
+        for (Iterator<? extends K> i = m.keySet().iterator(); i.hasNext();)
+            remove(i.next());
 
         // add all new values into this Map
-        this.groupingList.addAll(m.values());
+        groupingList.addAll(m.values());
     }
 
     /**
@@ -154,7 +172,7 @@ public class GroupingListMultiMap<K, V> implements Map<Comparable<K>, List<V>>,
      * @param value the value objects which should produce the given key when
      *      run through the key function
      */
-    private void checkKeyValueAgreement(Comparable<K> key, Collection<? extends V> value) {
+    private void checkKeyValueAgreement(K key, Collection<? extends V> value) {
         for (Iterator<? extends V> i = value.iterator(); i.hasNext();)
             checkKeyValueAgreement(key, i.next());
     }
@@ -168,8 +186,8 @@ public class GroupingListMultiMap<K, V> implements Map<Comparable<K>, List<V>>,
      * @param value the value object which should produce the given key when
      *      run through the key function
      */
-    private void checkKeyValueAgreement(Comparable<K> key, V value) {
-        final Comparable<K> k = key(value);
+    private void checkKeyValueAgreement(K key, V value) {
+        final K k = key(value);
 
         if (!GlazedListsImpl.equal(key, k))
             throw new IllegalArgumentException("The calculated key for the given value (" + k + ") does not match the given key (" + key + ")");
@@ -177,41 +195,56 @@ public class GroupingListMultiMap<K, V> implements Map<Comparable<K>, List<V>>,
 
     /** {@inheritDoc} */
     public void clear() {
-        this.groupingList.clear();
+        groupingList.clear();
     }
 
     /** {@inheritDoc} */
     public List<V> remove(Object key) {
-        final int index = this.keyList.indexOf(key);
-        return index == -1 ? null : this.groupingList.remove(index);
+        final int index = keyList.indexOf(key);
+        return index == -1 ? null : groupingList.remove(index);
     }
 
     /** {@inheritDoc} */
     public Collection<List<V>> values() {
-        return this.groupingList;
+        return groupingList;
     }
 
     /** {@inheritDoc} */
-    public Set<Comparable<K>> keySet() {
-        if (this.keySet == null)
-            this.keySet = new KeySet();
+    public Set<K> keySet() {
+        if (keySet == null)
+            keySet = new KeySet();
 
-        return this.keySet;
+        return keySet;
     }
 
     /** {@inheritDoc} */
-    public Set<Entry<Comparable<K>, List<V>>> entrySet() {
-        if (this.entrySet == null)
-            this.entrySet = new EntrySet();
+    public Set<Entry<K, List<V>>> entrySet() {
+        if (entrySet == null)
+            entrySet = new EntrySet();
 
-        return this.entrySet;
+        return entrySet;
+    }
+
+    /** @inheritDoc */
+    public boolean equals(Object o) {
+        return delegate.equals(o);
+    }
+
+    /** @inheritDoc */
+    public int hashCode() {
+        return delegate.hashCode();
     }
 
     /**
      * Updates this MultiMap datastructure to reflect changes in the underlying
      * {@link GroupingList}. Specifically, new entries are added to this
-     * MultiMap by calculating a key using key function this MultiMap was
-     * constructed with.
+     * MultiMap by calculating a key using the key function of this MultiMap.
+     *
+     * Interestingly, we don't have to handle the UPDATE events here. The
+     * entries in the delegate map are silenty updated in place since the List
+     * we were given by the GroupingList is simply mutated. INSERTS and
+     * DELETES, however, require actual changes to the delegate map, and thus
+     * are processed here accordingly.
      *
      * @param listChanges an event describing the changes in the GroupingList
      */
@@ -222,13 +255,13 @@ public class GroupingListMultiMap<K, V> implements Map<Comparable<K>, List<V>>,
 
             if (changeType == ListEvent.INSERT) {
                 final List<V> inserted = (List<V>) listChanges.getSourceList().get(changeIndex);
-                final Comparable<K> key = key(inserted);
-                this.keyList.add(changeIndex, key);
-                this.delegate.put(key, inserted);
+                final K key = key(inserted);
+                keyList.add(changeIndex, key);
+                delegate.put(key, inserted);
 
             } else if (changeType == ListEvent.DELETE) {
-                final Comparable<K> deleted = keyList.remove(changeIndex);
-                this.delegate.remove(deleted);
+                final K deleted = keyList.remove(changeIndex);
+                delegate.remove(deleted);
             }
         }
     }
@@ -240,7 +273,7 @@ public class GroupingListMultiMap<K, V> implements Map<Comparable<K>, List<V>>,
      *      {@link GroupingList} which share the same key value
      * @return the shared key which maps to each of the given values
      */
-    private Comparable<K> key(List<V> values) {
+    private K key(List<V> values) {
         return key(values.get(0));
     }
 
@@ -250,8 +283,8 @@ public class GroupingListMultiMap<K, V> implements Map<Comparable<K>, List<V>>,
      * @param value a single value from the source list
      * @return the key which maps to the given value
      */
-    private Comparable<K> key(V value) {
-        return this.keyFunction.evaluate(value);
+    private K key(V value) {
+        return keyFunction.evaluate(value);
     }
 
     /**
@@ -260,14 +293,14 @@ public class GroupingListMultiMap<K, V> implements Map<Comparable<K>, List<V>>,
      * "write through" to the backing {@link EventList} which ensures that both
      * the {@link EventList} and this MultiMap always remain in sync.
      */
-    private class EntrySet extends AbstractSet<Entry<Comparable<K>, List<V>>> {
+    private class EntrySet extends AbstractSet<Entry<K, List<V>>> {
         /** {@inheritDoc} */
         public int size() {
             return keyList.size();
         }
 
         /** {@inheritDoc} */
-        public Iterator<Entry<Comparable<K>, List<V>>> iterator() {
+        public Iterator<Entry<K, List<V>>> iterator() {
             return new EntrySetIterator(keyList.listIterator());
         }
 
@@ -276,8 +309,8 @@ public class GroupingListMultiMap<K, V> implements Map<Comparable<K>, List<V>>,
             if (!(o instanceof Map.Entry))
                 return false;
 
-            final Entry<Comparable<K>, List<V>> e = (Entry<Comparable<K>, List<V>>) o;
-            final Comparable<K> key = e.getKey();
+            final Entry<K, List<V>> e = (Entry<K, List<V>>) o;
+            final K key = e.getKey();
             final List<V> value = e.getValue();
 
             final List<V> mapValue = (List<V>) GroupingListMultiMap.this.get(key);
@@ -309,10 +342,10 @@ public class GroupingListMultiMap<K, V> implements Map<Comparable<K>, List<V>>,
      * {@link Map.Entry} object each time {@link #next} is called. Identity is
      * not preserved.
      */
-    private class EntrySetIterator implements Iterator<Entry<Comparable<K>, List<V>>> {
+    private class EntrySetIterator implements Iterator<Entry<K, List<V>>> {
 
         /** The delegate Iterator walks a List of keys for the MultiMap. */
-        private final ListIterator<Comparable<K>> keyIter;
+        private final ListIterator<K> keyIter;
 
         /**
          * Construct a new EntrySetIterator using a delegate Iterator that
@@ -320,7 +353,7 @@ public class GroupingListMultiMap<K, V> implements Map<Comparable<K>, List<V>>,
          *
          * @param keyIter a {@link ListIterator} that walks the keys of the MultiMap
          */
-        EntrySetIterator(ListIterator<Comparable<K>> keyIter) {
+        EntrySetIterator(ListIterator<K> keyIter) {
             this.keyIter = keyIter;
         }
 
@@ -332,8 +365,8 @@ public class GroupingListMultiMap<K, V> implements Map<Comparable<K>, List<V>>,
         /**
          * Returns a new {@link Map.Entry} each time this method is called.
          */
-        public Entry<Comparable<K>, List<V>> next() {
-            final Comparable<K> key = keyIter.next();
+        public Entry<K, List<V>> next() {
+            final K key = keyIter.next();
             return new MultiMapEntry(key, (List<V>) get(key));
         }
 
@@ -351,10 +384,10 @@ public class GroupingListMultiMap<K, V> implements Map<Comparable<K>, List<V>>,
      * "write through" to the backing {@link EventList} which ensures that
      * both the {@link EventList} and this MultiMap always remain in sync.
      */
-    private class MultiMapEntry implements Map.Entry<Comparable<K>, List<V>> {
+    private class MultiMapEntry implements Map.Entry<K, List<V>> {
 
         /** The MultiMap key for this Entry object. */
-        private final Comparable<K> key;
+        private final K key;
 
         /** The MultiMap value for this Entry object. */
         private List<V> value;
@@ -363,7 +396,7 @@ public class GroupingListMultiMap<K, V> implements Map<Comparable<K>, List<V>>,
          * Constructs a new MultiMapEntry with the given <code>key</code> and
          * initial <code>value</code>.
          */
-        MultiMapEntry(Comparable<K> key, List<V> value) {
+        MultiMapEntry(K key, List<V> value) {
             if (value == null) throw new IllegalArgumentException("value cannot be null");
 
             this.value = value;
@@ -371,7 +404,7 @@ public class GroupingListMultiMap<K, V> implements Map<Comparable<K>, List<V>>,
         }
 
         /** {@inheritDoc} */
-        public Comparable<K> getKey() {
+        public K getKey() {
             return key;
         }
 
@@ -393,7 +426,7 @@ public class GroupingListMultiMap<K, V> implements Map<Comparable<K>, List<V>>,
          */
         public List<V> setValue(List<V> newValue) {
             // ensure all of the newValue elements agree with the key of this Entry
-            checkKeyValueAgreement((Comparable<K>) getKey(), newValue);
+            checkKeyValueAgreement(getKey(), newValue);
 
             // record the old value List elements (to return)
             final List<V> oldValue = new ArrayList<V>(value);
@@ -439,14 +472,14 @@ public class GroupingListMultiMap<K, V> implements Map<Comparable<K>, List<V>>,
      * backing {@link EventList} which ensures that both the {@link EventList}
      * and this MultiMap always remain in sync.
      */
-    private class KeySet extends AbstractSet<Comparable<K>> {
+    private class KeySet extends AbstractSet<K> {
         /** {@inheritDoc} */
         public int size() {
             return keyList.size();
         }
 
         /** {@inheritDoc} */
-        public Iterator<Comparable<K>> iterator() {
+        public Iterator<K> iterator() {
             return new KeySetIterator(keyList.listIterator());
         }
 
@@ -472,10 +505,10 @@ public class GroupingListMultiMap<K, V> implements Map<Comparable<K>, List<V>>,
      * "write through" to the backing {@link EventList} which ensures that both
      * the {@link EventList} and this MultiMap always remain in sync.
      */
-    private class KeySetIterator implements Iterator<Comparable<K>> {
+    private class KeySetIterator implements Iterator<K> {
 
         /** The delegate Iterator walks a List of keys for the MultiMap. */
-        private final ListIterator<Comparable<K>> keyIter;
+        private final ListIterator<K> keyIter;
 
         /**
          * Construct a new KeySetIterator using a delegate Iterator that walks
@@ -483,7 +516,7 @@ public class GroupingListMultiMap<K, V> implements Map<Comparable<K>, List<V>>,
          *
          * @param keyIter a {@link ListIterator} that walks the keys of the MultiMap
          */
-        KeySetIterator(ListIterator<Comparable<K>> keyIter) {
+        KeySetIterator(ListIterator<K> keyIter) {
             this.keyIter = keyIter;
         }
 
@@ -493,7 +526,7 @@ public class GroupingListMultiMap<K, V> implements Map<Comparable<K>, List<V>>,
         }
 
         /** {@inheritDoc} */
-        public Comparable<K> next() {
+        public K next() {
             return keyIter.next();
         }
 
@@ -507,32 +540,33 @@ public class GroupingListMultiMap<K, V> implements Map<Comparable<K>, List<V>>,
 
     /**
      * This Comparator first runs each value through a
-     * {@link FunctionList.Function} to produce {@link Comparable} objects
-     * which are then compared to determine a relative ordering.
+     * {@link FunctionList.Function} to produce key objects which are then
+     * compared to determine a relative ordering using the given delegate
+     * {@link Comparator}.
      */
     private final class FunctionComparator implements Comparator<V> {
 
         /** A Comparator that orders {@link Comparable} objects. */
-        private final Comparator<Comparable> delegate = GlazedLists.comparableComparator();
+        private final Comparator<? super K> delegate;
 
         /** A function that extracts {@link Comparable} values from given objects. */
-        private final FunctionList.Function<V, ? extends Comparable<K>> function;
+        private final FunctionList.Function<V, ? extends K> function;
 
         /**
          * Construct a new FunctionComparator that uses the given
          * <code>function</code> to extract {@link Comparable} values from
          * given objects.
          */
-        FunctionComparator(FunctionList.Function<V, ? extends Comparable<K>> function) {
-            if (function == null) throw new IllegalArgumentException("function may not be null");
+        FunctionComparator(FunctionList.Function<V, ? extends K> function, Comparator<? super K> delegate) {
             this.function = function;
+            this.delegate = delegate;
         }
 
         /** {@inheritDoc} */
         public int compare(V o1, V o2) {
-            final Comparable c1 = function.evaluate(o1);
-            final Comparable c2 = function.evaluate(o2);
-            return delegate.compare(c1, c2);
+            final K k1 = function.evaluate(o1);
+            final K k2 = function.evaluate(o2);
+            return delegate.compare(k1, k2);
         }
     }
 
@@ -558,7 +592,7 @@ public class GroupingListMultiMap<K, V> implements Map<Comparable<K>, List<V>>,
         private final List<V> delegate;
 
         /** The key that all values in this List must share. */
-        private final Comparable<K> key;
+        private final K key;
 
         public ValueList(List<V> delegate) {
             this.delegate = delegate;
@@ -573,27 +607,27 @@ public class GroupingListMultiMap<K, V> implements Map<Comparable<K>, List<V>>,
         public <T>T[] toArray(T[] a) { return delegate.toArray(a); }
 
         public boolean add(V o) {
-            checkKeyValueAgreement(this.key, o);
+            checkKeyValueAgreement(key, o);
             return delegate.add(o);
         }
 
         public boolean addAll(Collection<? extends V> c) {
-            checkKeyValueAgreement(this.key, c);
+            checkKeyValueAgreement(key, c);
             return delegate.addAll(c);
         }
 
         public boolean addAll(int index, Collection<? extends V> c) {
-            checkKeyValueAgreement(this.key, c);
+            checkKeyValueAgreement(key, c);
             return delegate.addAll(index, c);
         }
 
         public void add(int index, V element) {
-            checkKeyValueAgreement(this.key, element);
+            checkKeyValueAgreement(key, element);
             delegate.add(index, element);
         }
 
         public V set(int index, V element) {
-            checkKeyValueAgreement(this.key, element);
+            checkKeyValueAgreement(key, element);
             return delegate.set(index, element);
         }
 
