@@ -4,15 +4,8 @@
 package ca.odell.glazedlists.impl.pmap;
 
 // NIO is used for CTP
-import ca.odell.glazedlists.impl.nio.*;
-import ca.odell.glazedlists.impl.io.*;
-import java.util.*;
-import java.nio.*;
-import java.nio.channels.*;
-import java.text.ParseException;
-import java.io.*;
-// logging
-import java.util.logging.*;
+import java.io.IOException;
+import java.util.logging.Logger;
 
 /**
  * Removes a chunk from the persistent map.
@@ -42,8 +35,6 @@ class RemoveChunk implements Runnable {
      * Removes the chunk by marking it off.
      */
     public void run() {
-        FileChannel fileChannel = persistentMap.getFileChannel();
-        
         try {
             chunk.delete();
             logger.info("Successfully removed value for key \"" + chunk.getKey() + "\"");
