@@ -3,13 +3,16 @@
 /*                                                     O'Dell Engineering Ltd.*/
 package ca.odell.glazedlists.swing;
 
-// the core Glazed Lists packages
-import ca.odell.glazedlists.*;
-// for listening to list selection events
+import ca.odell.glazedlists.EventList;
+import ca.odell.glazedlists.ListSelection;
+import ca.odell.glazedlists.TransformedList;
+import ca.odell.glazedlists.matchers.Matcher;
+
 import javax.swing.*;
-import javax.swing.event.*;
-// for lists of listeners
-import java.util.*;
+import javax.swing.event.ListSelectionEvent;
+import javax.swing.event.ListSelectionListener;
+import java.util.ArrayList;
+import java.util.List;
 
 /**
  * An {@link EventSelectionModel} is a class that performs two simulaneous
@@ -53,7 +56,7 @@ public final class EventSelectionModel<E> implements ListSelectionModel {
     private boolean enabled = true;
 
     /** listeners to notify when the selection changes */
-    private List<ListSelectionListener> listeners = new ArrayList<ListSelectionListener>();
+    private final List<ListSelectionListener> listeners = new ArrayList<ListSelectionListener>();
 
     /** whether there are a series of changes on the way */
     private boolean valueIsAdjusting = false;
@@ -105,10 +108,14 @@ public final class EventSelectionModel<E> implements ListSelectionModel {
      * the source list.
      */
     public EventList<E> getSelected() {
-        return listSelection.getSelected();
+        swingThreadSource.getReadWriteLock().readLock().lock();
+        try {
+            return listSelection.getSelected();
+        } finally {
+            swingThreadSource.getReadWriteLock().readLock().unlock();
+        }
     }
 
-
     /**
      * Gets an {@link EventList} that contains only selected
      * values and modifies the selection state on mutation.
@@ -118,7 +125,12 @@ public final class EventSelectionModel<E> implements ListSelectionModel {
      * {@link IllegalArgumentException} is thrown.
      */
     public EventList<E> getTogglingSelected() {
-        return listSelection.getTogglingSelected();
+        swingThreadSource.getReadWriteLock().readLock().lock();
+        try {
+            return listSelection.getTogglingSelected();
+        } finally {
+            swingThreadSource.getReadWriteLock().readLock().unlock();
+        }
     }
 
     /**
@@ -129,7 +141,12 @@ public final class EventSelectionModel<E> implements ListSelectionModel {
      * the source list.
      */
     public EventList<E> getDeselected() {
-        return listSelection.getDeselected();
+        swingThreadSource.getReadWriteLock().readLock().lock();
+        try {
+            return listSelection.getDeselected();
+        } finally {
+            swingThreadSource.getReadWriteLock().readLock().unlock();
+        }
     }
     
     /**
@@ -141,7 +158,12 @@ public final class EventSelectionModel<E> implements ListSelectionModel {
      * {@link IllegalArgumentException} is thrown
      */
     public EventList<E> getTogglingDeselected() {
-        return listSelection.getTogglingDeselected();
+        swingThreadSource.getReadWriteLock().readLock().lock();
+        try {
+            return listSelection.getTogglingDeselected();
+        } finally {
+            swingThreadSource.getReadWriteLock().readLock().unlock();
+        }
     }
 
     /**
@@ -168,7 +190,7 @@ public final class EventSelectionModel<E> implements ListSelectionModel {
     }
 
     /**
-     * Gets whether the EventSelectionModel is editable or not.
+     * Returns whether the EventSelectionModel is editable or not.
      */
     public boolean getEnabled() {
         return enabled;
@@ -179,7 +201,6 @@ public final class EventSelectionModel<E> implements ListSelectionModel {
      * {@link ListSelectionEvent}s to registered listeners.
      */
     private class SwingSelectionListener implements ListSelection.Listener {
-
         /** {@inheritDoc} */
         public void selectionChanged(int changeStart, int changeEnd) {
             fireSelectionChanged(changeStart, changeEnd);
@@ -200,11 +221,9 @@ public final class EventSelectionModel<E> implements ListSelectionModel {
         }
 
         // fire the change
-        ListSelectionEvent event = new ListSelectionEvent(this, changeStart, changeFinish, valueIsAdjusting);
-        for(Iterator<ListSelectionListener> i = listeners.iterator(); i.hasNext(); ) {
-            ListSelectionListener listener = i.next();
-            listener.valueChanged(event);
-        }
+        final ListSelectionEvent event = new ListSelectionEvent(this, changeStart, changeFinish, valueIsAdjusting);
+        for (int i = 0, n = listeners.size(); i < n; i++)
+            listeners.get(i).valueChanged(event);
     }
 
     /**
@@ -346,7 +365,12 @@ public final class EventSelectionModel<E> implements ListSelectionModel {
      * Returns true if no indices are selected.
      */
     public boolean isSelectionEmpty() {
-        return (listSelection.getSelected().size() == 0);
+        swingThreadSource.getReadWriteLock().readLock().lock();
+        try {
+            return listSelection.getSelected().isEmpty();
+        } finally {
+            swingThreadSource.getReadWriteLock().readLock().unlock();
+        }
     }
 
     /**
@@ -409,6 +433,26 @@ public final class EventSelectionModel<E> implements ListSelectionModel {
         return listSelection.getSelectionMode();
     }
 
+    /**
+     * Add a matcher which decides when source elements are valid for selection.
+     *
+     * @param validSelectionMatcher returns <tt>true</tt> if a source element
+     *      can be selected; <tt>false</tt> otherwise
+     */
+    public void addValidSelectionMatcher(Matcher<E> validSelectionMatcher) {
+        listSelection.addValidSelectionMatcher(validSelectionMatcher);
+    }
+
+    /**
+     * Remove a matcher which decides when source elements are valid for selection.
+     *
+     * @param validSelectionMatcher returns <tt>true</tt> if a source element
+     *      can be selected; <tt>false</tt> otherwise
+     */
+    public void removeValidSelectionMatcher(Matcher<E> validSelectionMatcher) {
+        listSelection.removeValidSelectionMatcher(validSelectionMatcher);
+    }
+
     /**
      * Add a listener to the list that's notified each time a change to
      * the selection occurs.
@@ -444,5 +488,6 @@ public final class EventSelectionModel<E> implements ListSelectionModel {
      */
     public void dispose() {
         listSelection.dispose();
+        swingThreadSource.dispose();
     }
 }
\ No newline at end of file
