@@ -3,64 +3,8 @@
 /*                                                     O'Dell Engineering Ltd.*/
 package ca.odell.glazedlists.util.concurrent;
 
-import java.util.Map;
 import java.util.HashMap;
-
-/**
- * An implementation of {@link LockFactory} that has been derived from Doug Lea's
- * <a href="http://gee.cs.oswego.edu/dl/classes/EDU/oswego/cs/dl/util/concurrent/intro.html">util.concurrent</a>.
- */
-final class J2SE12LockFactory implements LockFactory {
-
-    /**
-     * Create a {@link ReadWriteLock}.
-     *
-     * <p>The default implementation returns an implementation that has been
-     * derived from Doug Lea's <a href="http://gee.cs.oswego.edu/dl/classes/EDU/oswego/cs/dl/util/concurrent/intro.html">util.concurrent</a>.
-     */
-    public ReadWriteLock createReadWriteLock() {
-        return new J2SE12ReadWriteLock();
-    }
-
-    /**
-     * Create a {@link Lock}.
-     *
-     * <p>The default implementation returns an implementation that has been
-     * derived from Doug Lea's <a href="http://gee.cs.oswego.edu/dl/classes/EDU/oswego/cs/dl/util/concurrent/intro.html">util.concurrent</a>.
-     */
-    public Lock createLock() {
-        return new J2SE12ReadWriteLock().writeLock();
-    }
-}
-
-/**
- * A ReadWriteLock implementation that is compatible with Java 1.2 and better. This
- * is <a href="http://gee.cs.oswego.edu/dl/classes/EDU/oswego/cs/dl/util/concurrent/intro.html">Doug Lea's</a>
- * implementation, slightly modified to fit within the Glazed Lists codebase.
- *
- * @see <a href="http://gee.cs.oswego.edu/dl/classes/EDU/oswego/cs/dl/util/concurrent/intro.html">Introduction to util.concurrent</a>
- *
- * @author <a href="mailto:jesse@swank.ca">Jesse Wilson</a>
- */
-final class J2SE12ReadWriteLock implements ReadWriteLock {
-
-    /** delegate locking to the professionals! */
-    ReentrantWriterPreferenceReadWriteLock j2se12ReadWriteLock = new ReentrantWriterPreferenceReadWriteLock();
-
-    /**
-     * Return the lock used for reading.
-     */
-    public Lock readLock() {
-        return j2se12ReadWriteLock.readLock();
-    }
-
-    /**
-     * Return the lock used for writing.
-     */
-    public Lock writeLock() {
-        return j2se12ReadWriteLock.writeLock();
-    }
-}
+import java.util.Map;
 
 
 /*
@@ -142,7 +86,7 @@ class ReentrantWriterPreferenceReadWriteLock extends WriterPreferenceReadWriteLo
         Thread t = Thread.currentThread();
         Integer c = readers_.get(t);
         if (c != null) { // already held -- just increment hold count
-            readers_.put(t, new Integer(((Integer)(c)).intValue()+1));
+            readers_.put(t, new Integer(c.intValue()+1));
             ++activeReaders_;
             return true;
         }
@@ -182,7 +126,7 @@ class ReentrantWriterPreferenceReadWriteLock extends WriterPreferenceReadWriteLo
             throw new IllegalMonitorStateException("Attempted to unlock a readlock which was not locked. Please ensure the readlock is always locked and unlocked symmetrically.");
             --activeReaders_;
         if (c != IONE) { // more than one hold; decrement count
-            int h = ((Integer)(c)).intValue()-1;
+            int h = c.intValue()-1;
             Integer ih = (h == 1) ? IONE : new Integer(h);
             readers_.put(t, ih);
             return null;
